const _ = require('lodash');
const path = require('path');
const stylelint = require('stylelint');
const { report, ruleMessages, validateOptions } = stylelint.utils;

// Plugin name
const ruleName = 'sds-stylelint-plugin/component-pattern';

// Custom message for report
const messages = ruleMessages(ruleName, {
  expected: (prop, name) => `"${prop}" is not owned by the "${name}" component"`,
});

module.exports = stylelint.createPlugin(ruleName, function getPlugin(actual, options) {
  return function lint(postcssRoot, postcssResult) {
    const validOptions = validateOptions(
      postcssResult,
      ruleName,
      { actual },
      {
        actual: options,
        possible: {
          property: [_.isBoolean],
          value: [_.isBoolean],
        },
      },
    );

    if (!validOptions) {
      // If the options are invalid, don't lint
      return;
    }

    // Needed to parse out custom properties when used as fallbacks
    const customPropPattern = new RegExp(
      '(--[a-z0-9]*-(?<scope>[g|s|c])-(?<component>[a-z][a-z0-9]*)(-[a-z0-9]+)*)',
      'i',
    );

    if (options.property) {
      postcssRoot.walkDecls((decl) => {
        const file = path.parse(decl.source.input.file).name.split('.')[0];
        const prop = decl.prop;

        // This conditional deals with custom properties authored as a property
        // First check that the CSS property is a custom property
        if (prop.startsWith('--')) {
          const match = prop.match(customPropPattern);
          const scope = match.groups.scope;
          const component = match.groups.component;

          // Skip if global or shared scope
          if (scope === 'g' || scope === 's') {
            return;
          }

          /**
           * If property component scope and is authored inside the correct file then don't lint
           *
           * Two checks:
           * 1. Base - check the single-word component name matches the correct file
           * 2. Variants - split the '-' to deal with potential variants, checks against the first index which is the base component name
           */
          if (scope === 'c' && (file.split('-').join('') === component || file.split('-')[0] === component)) {
            return;
          }

          // Report custom properties that don't match valid pattern
          report({
            ruleName,
            result: postcssResult,
            message: messages.expected(prop, file), // Build the reported message
            node: decl, // Specify the reported node
          });
        }
      });
    }

    if (options.value) {
      postcssRoot.walkDecls((decl) => {
        const file = path.parse(decl.source.input.file).name.split('.')[0];
        const value = decl.value;
        // This conditional deals with custom properties authored as a value
        // First check that the value declaration starts with a var() function
        if (value.startsWith('var(')) {
          // Parse out CSS properties declared as a fallback value and store in array
          const allPropsAsValues = value.match(customPropPattern);
          const scope = allPropsAsValues.groups.scope;
          const component = allPropsAsValues.groups.component;
          // Then filter property out of array as a single value to run regex against
          const prop = allPropsAsValues.find((value) => customPropPattern.test(value));

          // Skip if global or shared scope
          if (scope === 'g' || scope === 's') {
            return;
          }

          /**
           * If property component scope and is authored inside the correct file then don't lint
           *
           * Two checks:
           * 1. Base - check the single-word component name matches the correct file
           * 2. Variants - split the '-' to deal with potential variants, checks against the first index which is the base component name
           */
          if (scope === 'c' && (file.split('-').join('') === component || file.split('-')[0] === component)) {
            return;
          }

          // Report custom properties that don't match valid pattern
          report({
            ruleName,
            result: postcssResult,
            message: messages.expected(prop, file), // Build the reported message
            node: decl, // Specify the reported node
          });
        }
      });
    }
  };
});

module.exports.ruleName = ruleName;
module.exports.messages = messages;
